@@ -57,13 +57,13 @@
 /************************************************
 
  ************************************************/
-QString expandCommand(const QString &command, QStringList *arguments=0)
+static QString expandCommand(const QString &command, QStringList *arguments=0)
 {
     QString program;
     wordexp_t words;
 
     if (wordexp(command.toLocal8Bit().data(), &words, 0) != 0)
-        return "";
+        return QString();
 
     char **w;
     w = words.we_wordv;
@@ -83,14 +83,52 @@ QString expandCommand(const QString &command, QStringList *arguments=0)
 /************************************************
 
  ************************************************/
-bool startProcess(QString command)
+static QString which(const QString &progName)
+{
+    if (progName.isEmpty())
+        return QString();
+
+    if (progName.startsWith(QDir::separator()))
+    {
+        QFileInfo fileInfo(progName);
+        if (fileInfo.isExecutable() && fileInfo.isFile())
+            return fileInfo.absoluteFilePath();
+    }
+
+    const QStringList dirs = QString(getenv("PATH")).split(":");
+
+    foreach (const QString &dir, dirs)
+    {
+        QFileInfo fileInfo(QDir(dir), progName);
+        if (fileInfo.isExecutable() && fileInfo.isFile())
+            return fileInfo.absoluteFilePath();
+    }
+
+    return QString();
+}
+
+
+/************************************************
+
+ ************************************************/
+static bool startProcess(QString command)
 {
     QStringList args;
     QString program  = expandCommand(command, &args);
     if (program.isEmpty())
         return false;
+    if (QProcess::startDetached(program, args))
+    {
+        return true;
+    } else
+    {
+        //fallback for executable script with no #!
+        //trying as in system(2)
+        args.prepend(program);
+        args.prepend(QStringLiteral("-c"));
+        return QProcess::startDetached(QStringLiteral("/bin/sh"), args);
+    }
 
-    return QProcess::startDetached(program, args);
 }
 
 
@@ -143,6 +181,7 @@ AppLinkItem::AppLinkItem(const QDomElement &element):
     mCommand = element.attribute("exec");
     mProgram = QFileInfo(element.attribute("exec")).baseName().section(" ", 0, 0);
     mDesktopFile = element.attribute("desktopFile");
+    initExec();
     QMetaObject::invokeMethod(this, "updateIcon", Qt::QueuedConnection);
 }
 
@@ -160,6 +199,7 @@ AppLinkItem::AppLinkItem(MenuCacheApp* app):
     char* path = menu_cache_item_get_file_path(MENU_CACHE_ITEM(app));
     mDesktopFile = QString::fromLocal8Bit(path);
     g_free(path);
+    initExec();
     QMetaObject::invokeMethod(this, "updateIcon", Qt::QueuedConnection);
     // qDebug() << "FOUND: " << mIconName << ", " << mCommand;
 }
@@ -191,6 +231,7 @@ void AppLinkItem::operator=(const AppLinkItem &other)
     mCommand = other.mCommand;
     mProgram = other.mProgram;
     mDesktopFile = other.mDesktopFile;
+    mExec = other.mExec;
 
     mIconName = other.mIconName;
     mIcon = other.icon();
@@ -230,14 +271,25 @@ bool AppLinkItem::compare(const QRegExp &regExp) const
     if (regExp.isEmpty())
         return false;
 
-    QRegExp re(regExp);
-
-    re.setCaseSensitivity(Qt::CaseInsensitive);
-    return mProgram.contains(re) ||
-           mTitle.contains(re) ;
+    return mProgram.contains(regExp) ||
+           mTitle.contains(regExp) ;
 }
 
 
+/************************************************
+
+ ************************************************/
+void AppLinkItem::initExec()
+{
+    static const QRegExp split_re{QStringLiteral("\\s")};
+    XdgDesktopFile desktop;
+    if (desktop.load(mDesktopFile))
+    {
+        QStringList cmd = desktop.value(QStringLiteral("Exec")).toString().split(split_re);
+        if (0 < cmd.size())
+            mExec = which(expandCommand(cmd[0]));
+    }
+}
 
 
 /************************************************
@@ -255,7 +307,7 @@ AppLinkProvider::AppLinkProvider():
         mMenuCacheNotify = 0;
 #else
     mXdgMenu = new XdgMenu();
-    mXdgMenu->setEnvironments(QStringList() << "X-LXQT" << "LxQt");
+    mXdgMenu->setEnvironments(QStringList() << "X-LXQT" << "LXQt");
     connect(mXdgMenu, SIGNAL(changed()), this, SLOT(update()));
     mXdgMenu->read(XdgMenu::getMenuFileName());
     update();
@@ -295,7 +347,7 @@ void AppLinkProvider::menuCacheReloadNotify(MenuCache* cache, gpointer user_data
 
  void doUpdate(const QDomElement &xml, QHash<QString, AppLinkItem*> &items)
 {
-    DomElementIterator it(xml, "");
+    DomElementIterator it(xml, QString());
     while (it.hasNext())
     {
         QDomElement e = it.next();
@@ -355,7 +407,7 @@ void AppLinkProvider::update()
                 // Otherwise the copied item will have no icon.
                 // FIXME: this is a dirty hack and it should be made cleaner later.
                 if(item->icon().isNull())
-		    QMetaObject::invokeMethod(item, "updateIcon", Qt::QueuedConnection);
+                    QMetaObject::invokeMethod(item, "updateIcon", Qt::QueuedConnection);
                 delete newItem;
             }
             else
@@ -407,10 +459,7 @@ bool HistoryItem::run() const
  ************************************************/
 bool HistoryItem::compare(const QRegExp &regExp) const
 {
-    QRegExp re(regExp);
-
-    re.setCaseSensitivity(Qt::CaseSensitive);
-    return mCommand.contains(re);
+    return mCommand.contains(regExp);
 }
 
 
@@ -499,34 +548,6 @@ CustomCommandItem::CustomCommandItem(CustomCommandProvider *provider):
 }
 
 
-/************************************************
-
- ************************************************/
-QString which(const QString &progName)
-{
-    if (progName.isEmpty())
-        return "";
-
-    if (progName.startsWith(QDir::separator()))
-    {
-        QFileInfo fileInfo(progName);
-        if (fileInfo.isExecutable() && fileInfo.isFile())
-            return fileInfo.absoluteFilePath();
-    }
-
-    QStringList dirs = QString(getenv("PATH")).split(":");
-
-    foreach (QString dir, dirs)
-    {
-        QFileInfo fileInfo(QDir(dir), progName);
-        if (fileInfo.isExecutable() && fileInfo.isFile())
-            return fileInfo.absoluteFilePath();
-    }
-
-    return "";
-}
-
-
 /************************************************
 
  ************************************************/
@@ -535,12 +556,12 @@ void CustomCommandItem::setCommand(const QString &command)
     mCommand = command;
     mTitle = mCommand;
 
-    QString program  = which(expandCommand(command));
+    mExec = which(expandCommand(command));
 
-    if (!program.isEmpty())
-        mComment = QString("%1 %2").arg(program, command.section(' ', 1));
+    if (!mExec.isEmpty())
+        mComment = QString("%1 %2").arg(mExec, command.section(' ', 1));
     else
-        mComment = "";
+        mComment = QString();
 
 }
 
@@ -597,13 +618,13 @@ VirtualBoxItem::VirtualBoxItem(const QString & MachineName , const QIcon & Icon)
 
 void VirtualBoxItem::setRDEPort(const QString & portNum)
 {
-	m_rdePortNum = portNum;
+    m_rdePortNum = portNum;
 }
 
 bool VirtualBoxItem::run() const
 {
     QStringList arguments;
-#ifdef VBOXRUNNER_HEADLESS
+#ifdef VBOX_HEADLESS_ENABLED
     arguments << "-startvm" << title();
     return QProcess::startDetached ("VBoxHeadless" , arguments);
 #else
@@ -615,9 +636,7 @@ bool VirtualBoxItem::run() const
 
 bool VirtualBoxItem::compare(const QRegExp &regExp) const
 {
-    QRegExp re(regExp);
-    re.setCaseSensitivity(Qt::CaseInsensitive);
-    return (! regExp.isEmpty() && -1 != re.indexIn (title ()));
+    return (! regExp.isEmpty() && -1 != regExp.indexIn (title ()));
 }
 
 unsigned int VirtualBoxItem::rank(const QString &pattern) const
@@ -770,20 +789,49 @@ bool VirtualBoxProvider::isOutDated() const
 
 
 #ifdef MATH_ENABLED
-#include <QtScript/QScriptEngine>
-#include <QtScript/QScriptValue>
+#include <muParser.h>
+
+class MathItem::Parser : public mu::Parser
+{
+public:
+    static void initLocale()
+    {
+        try
+        {
+            // use the system's locale instead of the "C"
+            s_locale = std::locale{""};
+        } catch (const std::runtime_error & e)
+        {
+            qWarning().noquote() << "Unable to set locale for Math, " << e.what();
+        }
+    }
+};
+static void muParserInitLocale()
+{
+    MathItem::Parser::initLocale();
+}
+Q_COREAPP_STARTUP_FUNCTION(muParserInitLocale);
 
 /************************************************
 
  ************************************************/
 MathItem::MathItem():
-        CommandProviderItem()
+        CommandProviderItem(),
+        mParser{new Parser}
 {
     mToolTip =QObject::tr("Mathematics");
     mIcon = XdgIcon::fromTheme("accessories-calculator");
 }
 
 
+/************************************************
+
+ ************************************************/
+MathItem::~MathItem()
+{
+}
+
+
 /************************************************
 
  ************************************************/
@@ -800,17 +848,42 @@ bool MathItem::compare(const QRegExp &regExp) const
 {
     QString s = regExp.pattern().trimmed();
 
+    bool is_math = false;
+    if (s.startsWith('='))
+    {
+        is_math = true;
+        s.remove(0, 1);
+    }
     if (s.endsWith("="))
     {
+        is_math = true;
         s.chop(1);
-        QScriptEngine myEngine;
-        QScriptValue res = myEngine.evaluate(s);
-        if (res.isNumber())
+    }
+
+    if (is_math)
+    {
+        if (s != mCachedInput)
         {
-            MathItem *self=const_cast<MathItem*>(this);
-            self->mTitle = s + " = " + res.toString();
-            return true;
+            MathItem * self = const_cast<MathItem*>(this);
+            mCachedInput = s;
+            self->mTitle.clear();
+
+            //try to compute anything suitable
+            for (int attempts = 20; 0 < attempts && 0 < s.size(); s.chop(1), --attempts)
+            {
+                try
+                {
+                    mParser->SetExpr(s.toStdString());
+                    self->mTitle = s + "=" + QLocale::system().toString(mParser->Eval());
+                    break;
+                } catch (const mu::Parser::exception_type & e)
+                {
+                    //don't do anything, return false -> no result will be showed
+                }
+            }
         }
+
+        return !mTitle.isEmpty();
     }
 
     return false;
@@ -833,53 +906,7 @@ MathProvider::MathProvider()
     append(new MathItem());
 }
 
-
-PowerProviderItem::PowerProviderItem(QAction *action)
-    : CommandProviderItem(),
-      m_action(action)
-{
-    mIcon = action->icon();
-    mTitle = action->text();
-    mComment = QObject::tr("Power Management");
-    mToolTip = mComment;
-}
-
-bool PowerProviderItem::run() const
-{
-    m_action->activate(QAction::Trigger);
-    return true;
-}
-
-bool PowerProviderItem::compare(const QRegExp &regExp) const
-{
-    if (regExp.isEmpty())
-        return false;
-
-    QRegExp re(regExp);
-
-    re.setCaseSensitivity(Qt::CaseInsensitive);
-    return mTitle.contains(re) ;
-}
-
-unsigned int PowerProviderItem::rank(const QString &pattern) const
-{
-    return stringRank(mTitle, pattern);
-}
-
-PowerProvider::PowerProvider() : CommandProvider()
-{
-    m_power = new LxQt::PowerManager(this);
-    foreach (QAction *a, m_power->availableActions())
-    {
-        append(new PowerProviderItem(a));
-    }
-
-    m_screensaver = new LxQt::ScreenSaver(this);
-    foreach (QAction *a, m_screensaver->availableActions())
-    {
-        append(new PowerProviderItem(a));
-    }
-}
+#endif
 
 ExternalProviderItem::ExternalProviderItem()
 {
@@ -970,4 +997,3 @@ void ExternalProvider::readFromProcess()
     }
 }
 
-#endif
